{
  open Defs          
  open Parser        (* The type token is defined in parser.mli *)
  exception Eof

  let keyword_table = Hashtbl.create 53
  let _ =
    List.iter (fun (kwd, tok) -> Hashtbl.add keyword_table kwd tok)
              [ "read", READ;
                "write", WRITE;
                "begin", BEGIN;
                "end", END
              ]
}

let digit  = ['0'-'9']           
let letter = ['a'-'z''A'-'Z']
let number = digit+
let name   = letter(letter|digit)*

rule token = parse
     [' ' '\t' '\n'] { token lexbuf }  (* skip white spaces *)
   | number as lxm   { INT (int_of_string lxm) }
   | name   as lxm   { try Hashtbl.find keyword_table (String.lowercase lxm)
                       with Not_found -> VAR lxm }
   | '+'             { PLUS }
   | '-'             { MINUS }
   | '*'             { TIMES }
   | '/'             { DIV }
   | '('             { LPAREN }
   | ')'             { RPAREN }
   | ';'             { SEMICOLON }
   | ":="            { ASSIGN }
   | "read"          { READ }
   | "write"         { WRITE }
   | eof             { raise Eof }
