open Defs
open Printf

let rec pprint_Expression = function
    CInt i -> printf "%d" i
  | Var var -> printf "%s" var    
  | Op (sym,[a]) -> 
      printf "%s" sym;
      pprint_Expression a
  | Op (sym,[a;b]) ->
      printf "(";
      pprint_Expression a;
      printf "%s" sym;
      pprint_Expression b;
      printf ")"

let pprint_Statement = function
    Assign (var,exp) -> 
      printf "%s := " var;
      pprint_Expression exp
  | Read var -> 
      printf "read %s" var 
  | Write exp -> 
      printf "write ";
      pprint_Expression exp

let pprint_Program = function
    Program (x::xs) -> 
      printf "begin\n";
      List.fold_left (fun _ s -> printf ";\n"; pprint_Statement s)
                     (pprint_Statement x)
                     xs;
      printf "\nend\n"


let _ = 
  let prg = ref (Program []) in
  try
    let lexbuf = Lexing.from_channel stdin in
    prg := Parser.main Lexer.token lexbuf;
    pprint_Program !prg
  with _ -> printf "parse error\n"

